# coding: utf-8
import threading

class MultiThread():
    """ 多线程
    Arguments:
      items: 配置列表
      func: 根据配置返回target及args, 可返回字典或数组, 如果是数组则第一个值为方法第二个值为入参
    Attributes:
      start: 开启多线程任务
    """
    def __init__(self, items, func):
        self.threads = []
        for idx, item in enumerate(items):
            params = func(item, idx)
            args = params if type(params) == dict else {}
            if type(params) == list:
                args['target'], args['args'] = params
            self.threads.append(threading.Thread(**args))
    
    def start(self):
        for i in self.threads: i.start()
        for i in self.threads: i.join()



# coding: utf-8
import sunday
from os import path, makedirs
from sunday.core.getEnv import getEnv
from dotenv import load_dotenv

__all__ = ['rootCwd', 'logCwd', 'binCwd', 'envCwd', 'moduleLockCwd', 'sundayLoginCwd', 'sundayToolsCwd']

# sunday家目录
userHomeCwd = path.expanduser('~')
rootCwd = getEnv('SUNDAY_ROOT') or path.join(userHomeCwd, '.sunday')
homePluginsCwd = path.join(rootCwd, 'plugins')
binCwd = path.join(rootCwd, 'bin')
envCwd = path.join(rootCwd, '.env')
logCwd = path.join(rootCwd, 'log')
moduleLockCwd = path.join(rootCwd, 'module-lock.json')
path.exists(envCwd) and load_dotenv(envCwd)

for p in [binCwd, logCwd, homePluginsCwd]:
    if not path.exists(p): makedirs(p)

# sunday模块目录
#pathsCwd = path.realpath(__file__)
#sundayCwd = path.abspath(path.join(pathsCwd, '../..'))
sundayCwd = path.dirname(sunday.__file__)
sundayLoginCwd = path.join(sundayCwd, 'login')
sundayToolsCwd = path.join(sundayCwd, 'tools')

# 配置文件
configCwd = path.join(rootCwd, 'config.ini')
if not path.exists(configCwd): configCwd = path.join(sundayCwd, 'config.ini')



# coding: utf-8
import sunday.core.paths as paths
import atexit
import time
import stat
from sunday.core.common import parseJson
from os import path, remove, symlink, listdir, readlink, access, X_OK, chmod
from sunday.core.inner import grenLoginAndToolsInit
from sunday.core.globalvar import getvar, setvar
from sunday.core.logger import Logger
from sunday.core.globalKeyMaps import sdvar_premust, sdvar_logger, sdvar_exectime

__all__ = []

def checkModuleExist():
    # 核对插件存在情况
    changeFlag = False
    logger = getvar(sdvar_logger)
    for item in parseJson(paths.moduleLockCwd, [], ['origin', 'target', 'type', 'name']):
        target = item['target']
        origin = item['origin']
        mname = item['name']
        mtype = item['type']
        hasTar = path.islink(target)
        hasOri = path.exists(origin)
        if hasTar and not hasOri:
            # 删除软链
            remove(target)
            changeFlag = True
            logger.warning('插件不存在软链存在, 删除软链 %s/%s' % (mtype, mname))
        elif hasOri and not hasTar:
            # 建立软链
            symlink(origin, target)
            changeFlag = True
            logger.warning('插件存在软链不存在, 创建软链 %s/%s' % (mtype, mname))
    if changeFlag:
        grenLoginAndToolsInit()

def checkCommandExist():
    # 核对命令软链是否合法
    cmdList = filter(path.islink, map(lambda name:
        path.join(paths.binCwd, name), listdir(paths.binCwd)))
    for cmd in cmdList:
        target = readlink(cmd)
        if not path.exists(target):
            remove(cmd)
            logger.warning('命令不存在软链存在, 删除软链 %s' % path.basename(cmd))
        elif not access(target, X_OK):
            # 原文件无权限则添加执行权限
            chmod(target, stat.S_IRWXU)

def initPrintTime():
    now = time.time()
    vars = { 'firstTime': now, 'preTime': now }
    def printTime():
        nowTime = time.time()
        lastExecTime = nowTime - vars['preTime']
        totalExecTime = nowTime - vars['firstTime']
        vars['preTime'] = nowTime
        return lastExecTime, totalExecTime
    return printTime

if not getvar(sdvar_premust):
    setvar(sdvar_premust, True)
    setvar(sdvar_logger, Logger('SUNDAY').getLogger())
    setvar(sdvar_exectime, initPrintTime())

    logger = getvar(sdvar_logger)
    exectime = getvar(sdvar_exectime)

    checkModuleExist()
    checkCommandExist()
    @atexit.register
    def exitPrintExecInfo():
        _, totalExecTime = exectime()
        logger.info('program execution time %.2f s' % totalExecTime)



# coding: utf-8
from prettytable import PrettyTable

def printTable(titleArr=[], align='', isPrint=True):
    '''
    用于表格打印
        titleArr为表格头
        align为对齐方式
        isPrint为是否打印
    '''
    table = PrettyTable(titleArr)
    if align:
        for title in titleArr:
            table.align[title] = align
    def log(rowArr=[], alone=False):
        if alone: rowArr = [rowArr]
        for row in rowArr:
            table.add_row(row)
        if isPrint: print(table)
        return table
    return log



# coding: utf-8
import os
import requests
from http.cookiejar import LWPCookieJar, CookieJar
from sunday.core.getConfig import getConfig
from sunday.core.logger import Logger
from sunday.core.fetch import Fetch
import sunday.core.globalvar as globalvar
from sunday.utils.tools import mergeObj
from pydash import get


class LoginBase():
    def __init__(self, file, logger=Logger('LoginBase').getLogger(), pacWifi=None, pacUrl=None, ident=''):
        '''
        file: 目标登录对象执行文件
        logger: 日志对象，用于日志打印
        pacWifi: 匹配的wifi开启pac代理
        pacUrl: 匹配的url开启pac代理
        ident: 多用户标识，用于多用户认证
        '''
        pwd = os.path.dirname(os.path.abspath(file))
        self.logger = logger
        cfg = getConfig('LOGIN')
        ident = ('-' if ident else '') + ident
        self.cookiePwd = os.path.join(pwd, cfg('cookieFile') + ident)
        self.envPwd = os.path.join(pwd, cfg('envFile') + ident)
        self.fetch = Fetch(pacWifi=pacWifi, pacUrl=pacUrl)

    def getCookiePwd(self):
        return self.cookiePwd

    def getEnvPwd(self):
        return self.envPwd

    def saveCookie(self):
        '''保存cookie'''
        savefun = get(self.fetch, 'session.cookies.save')
        if callable(savefun): savefun(ignore_discard=True, ignore_expires=True)

    def getCookie(self, useHistory=True):
        """ params(useHistory): 控制是否使用历史登陆态
            return(cookies, boolean): 返回cookie对象和是否登录标志
        """
        filename = self.getCookiePwd()
        hasCookieFile = os.path.isfile(filename)
        if useHistory and hasCookieFile:
            load_cookiejar = LWPCookieJar()
            load_cookiejar.load(filename, ignore_discard=True, ignore_expires=True)
            load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
            self.logger.debug('是否存在登录态: 是')
            return requests.utils.cookiejar_from_dict(load_cookies), True
        if hasCookieFile: os.remove(filename)
        self.logger.debug('是否存在登录态: 否')
        return LWPCookieJar(filename=filename), False

    def setCookie(self, name, value, domain, rest={}, **argvs):
        '''增加新的cookie, 入参参考cookielib.Cookie中的定义'''
        cookies = self.fetch.session.cookies
        standard = mergeObj({ 'path': '/', 'domain': domain, 'version': 0 }, argvs)
        cookie = CookieJar()._cookie_from_cookie_tuple((name, value, standard, rest), None)
        cookies.set_cookie(cookie)
    
    def checkLogin(self, checkUrl):
        '''确认是否登录成功, 方法: 请求首页, 如果跳到登录页则说明登录态失效'''
        # 服务端存在缓存因此请求两次
        self.fetch.get(checkUrl)
        res = self.fetch.get(checkUrl)
        return res.status_code != 200 or res.url != checkUrl

    def initRs(self, checkUrl, useHistory=True):
        """
        checkUrl: 检查登录的链接
        useHistory: 是否使用历史，为False则为重新登录不用历史登录态
        """
        session = self.fetch.session
        session.cookies, isLogin = self.getCookie(useHistory=useHistory)
        if isLogin and self.checkLogin(checkUrl):
            session.cookies, isLogin = self.getCookie(useHistory=False)
        return self.fetch, isLogin




import base64
from Crypto.Cipher import AES
from Crypto import Random
from hashlib import md5

__all__ = ['aesCbcDecrypt', 'aesCbcEncrypt']

def pad(s):
    return s + (16 - len(s) % 16) * chr(16 - len(s) % 16).encode()

def unpad(s):
    return s[0:-ord(s[len(s)-1:])]

def bytes_to_key(data, salt, output=48):
    assert len(salt) == 8, len(salt)
    data += salt
    key = md5(data).digest()
    final_key = key
    while len(final_key) < output:
        key = md5(key + data).digest()
        final_key += key
    return final_key[:output]

def aesCbcEncrypt(data, passphrase):
    if type(data) == str: data = data.encode()
    if type(passphrase) == str: passphrase = passphrase.encode()
    salt = Random.new().read(8)
    key_iv = bytes_to_key(passphrase, salt, 32+16)
    key = key_iv[:32]
    iv = key_iv[32:]
    aes = AES.new(key, AES.MODE_CBC, iv)
    cipherbyte = base64.b64encode(b"Salted__" + salt + aes.encrypt(pad(data)))
    return cipherbyte.decode()

def aesCbcDecrypt(data, passphrase):
    if type(data) == str: data = data.encode()
    if type(passphrase) == str: passphrase = passphrase.encode()
    data = base64.b64decode(data)
    assert data[:8] == b'Salted__'
    salt = data[8:16]
    key_iv = bytes_to_key(passphrase, salt, 32+16)
    key = key_iv[:32]
    iv = key_iv[32:]
    aes = AES.new(key, AES.MODE_CBC, iv)
    plainbyte = unpad(aes.decrypt(data[16:]))
    return plainbyte.decode()

if __name__ == '__main__':
    data = b'8879576hdud'
    passphrase = b'HOWDUUDU'

    encrypt_data = aesCbcEncrypt(data, passphrase)
    print('encrypt_data:', encrypt_data)

    encrypt_data = b"U2FsdGVkX1+N4vlBhSYzxlN075knzFQk4l4VsxT71h0="

    decrypt_data = aesCbcDecrypt(encrypt_data, passphrase)
    print('decrypt_data:', decrypt_data)




from optparse import OptionParser

'''传入命令参数数组, 初始化命令参数
    short: string
    full: string
    action : string
    type : string
    dest : string
    default : any
    nargs : int
    const : any
    choices : [string]
    callback : function
    callback_args : (any*)
    callback_kwargs : { string : any }
    help : string
    metavar : string
'''
def initOptions(optionArr):
    parser = OptionParser()
    if not opt['short']: raise Exception('请给简称(short)')
    if not opt['full']: raise Exception('请给全称(full)')
    for opt in optionArr:
        parser.add_option(opt['short'], opt['full'], **opt)
    return parser.parse_args()



# coding: utf-8
import os
import sys
import re
import json
from pydash import pick as _pick
from sunday.core.globalvar import getvar
from sunday.core.globalKeyMaps import sdvar_logger

def clear():
    """清理屏幕"""
    os.system('clear')

def exit(str = ''):
    """退出程序"""
    if str: getvar(sdvar_logger).error(str)
    sys.exit(1)

def pascal_to_snake(camel_case):
    """驼峰转蛇形"""
    snake_case = re.sub(r"(?P<key>[A-Z])", r"_\g<key>", camel_case)
    return snake_case.lower().strip('_')

def snake_to_pascal(snake_case):
    """蛇形转驼峰"""
    words = snake_case.split('_')
    return ''.join(word.title() for word in words)

def parseJson(jsonPath, defaultValue={}, keywords=[], logger=None):
    """传入json文件，返回解析后的数据"""
    if not logger: logger = getvar(sdvar_logger)
    if not os.path.exists(jsonPath): return defaultValue
    with open(jsonPath) as f:
        try:
            content = json.load(f)
            if type(defaultValue) == dict:
                return _pick(content, keywords)
            elif type(defaultValue) == list:
                return [_pick(cont, keywords) for cont in content]
        except Exception as e:
            logger.error('读取文件失败(%s): %s' % (jsonPath, e))
        return defaultValue



#!/usr/bin/env python
# coding: utf-8

import stat
import pdb
from os import path, chmod, listdir, system
from setuptools import setup
from setuptools.command.install import install

version = '0.2.8.8'

binCwd = path.join(path.dirname(path.realpath(__file__)), 'bin')
if path.exists(binCwd):
    for name in listdir(binCwd): chmod(path.join(binCwd, name), stat.S_IRWXU)

with open('./README.md', 'r') as f:
    readme = f.read()

install_requires = []
with open("requirements.txt", "r") as fp:
    for line in fp:
        line.strip()
        if line:
            install_requires.append(line.strip())

class Install(install):
    def run(self):
        install.run(self)
        system('pip3 install -r requirements.txt')
        print('''
                          #
  mmm   m   m  m mm    mmm#   mmm   m   m
 #   "  #   #  #"  #  #" "#  "   #  "m m"
  """m  #   #  #   #  #   #  m"""#   #m#
 "mmm"  "mm"#  #   #  "#m##  "mm"#   "#
                                     m"
                                    ""

安装成功, sunday让生活更美好!

自定义目录配置:
    export SUNDAY_ROOT=~/.sunday
    export PATH=$SUNDAY_ROOT/bin:$PATH
默认目录配置:
    export PATH=~/.sunday/bin:$PATH
''')

setup(
    name='pysunday',
    version=version,
    description='一款目标明确的敏捷开发工具',
    long_description_content_type='text/markdown',
    long_description=readme,
    author='ct',
    author_email='it17621000@163.com',
    python_requires='>3',
    packages=['sunday', 'sunday.core', 'sunday.utils', 'sunday.login', 'sunday.tools'],
    package_dir={'sunday': 'src'},
    install_requires=install_requires,
    extras_require={
        'debug': ['ipython', 'ipdb']
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
    ],
    scripts=['bin/sunday_install'],
    dependency_links=[
        'https://pypi.org/simple/'
    ],
    cmdclass={
        'install': Install,
    }
)



# coding: utf-8

if globals().get('_global_var') is None:
    global _global_var
    _global_var = {}

def getvar(name):
    '''返回全局变量值'''
    return _global_var.get(name)

def setvar(name, value):
    '''设置全局变量值'''
    _global_var[name] = value
